<?php
/**
 * Created by PhpStorm.
 * User: RITG (http://ritg.ru)
 * Date: 23.04.2018
 * Time: 13:06
 */

use Bitrix\Main\Localization\Loc;
use Bitrix\Main\ModuleManager;
use Bitrix\Main\Config\Option;
use Bitrix\Main\EventManager;
use Bitrix\Main\Application;
use Bitrix\Main\IO\Directory;

Loc::loadMessages(__FILE__);

class litebox_kassa extends CModule
{
    public $MODULE_ID = 'litebox.kassa';
    public $MODULE_NAME = 'litebox.kassa';
    public $pathResources = 'local';
    public $componentPath = '/components/litebox/kassa';
    public $apiPath = '/api';

    public $urlRules = [
        "CONDITION" => '#^/api/#',
        "ID" => 'litebox:kassa',
        "PATH" => '/api/index.php',
        "RULE" => '',
    ];

    public function __construct()
    {
        if(file_exists(__DIR__."/version.php")){

            $arModuleVersion = array();
			
			$path = str_replace("\\", "/", __FILE__);
			$path = substr($path, 0, strlen($path) - strlen("/index.php"));
			include($path . "/version.php");

            $this->MODULE_ID = str_replace("_", ".", get_class($this));
            $this->MODULE_VERSION = $arModuleVersion["VERSION"];
            $this->MODULE_VERSION_DATE = $arModuleVersion["VERSION_DATE"];
            $this->MODULE_NAME = Loc::getMessage("LITEBOX_SYNC_NAME");
            $this->MODULE_DESCRIPTION = Loc::getMessage("LITEBOX_SYNC_DESCRIPTION");
            $this->PARTNER_NAME = Loc::getMessage("LITEBOX_SYNC_PARTNER_NAME");
            $this->PARTNER_URI = Loc::getMessage("LITEBOX_SYNC_PARTNER_URI");
        }

        return false;
    }

    public function DoInstall()
    {
        global $APPLICATION;

        if(CheckVersion(ModuleManager::getVersion("main"), "14.00.00")){

            $this->InstallFiles();
            $this->InstallRouter();

            ModuleManager::registerModule($this->MODULE_ID);

	        $this->InstallDB();
            $this->InstallEvents();
        }else{

            $APPLICATION->ThrowException(
                Loc::getMessage("LITEBOX_SYNC_INSTALL_ERROR_VERSION")
            );
        }

        $APPLICATION->IncludeAdminFile(
            Loc::getMessage("LITEBOX_SYNC_INSTALL_TITLE")." \"".Loc::getMessage("LITEBOX_SYNC_NAME")."\"", __DIR__."/step.php"
        );

        return false;
    }

    public function DoUninstall()
    {
        global $APPLICATION;

        $this->UnInstallFiles();
        $this->UnInstallRouter();
        $this->UnInstallDB();

        ModuleManager::unRegisterModule($this->MODULE_ID);

        $APPLICATION->IncludeAdminFile(
            Loc::getMessage("LITEBOX_SYNC_UNINSTALL_TITLE") . " \"" . Loc::getMessage("LITEBOX_SYNC_NAME") . "\"",
            __DIR__ . "/unstep.php"
        );

        return false;
    }

    public function UnInstallFiles()
    {
        \Bitrix\Main\IO\Directory::deleteDirectory($_SERVER["DOCUMENT_ROOT"] . $this->apiPath);
        \Bitrix\Main\IO\Directory::deleteDirectory($_SERVER["DOCUMENT_ROOT"] . "/" . $this->pathResources . $this->componentPath);
    }

    public function UnInstallDB()
    {
	    \Bitrix\Main\Loader::IncludeModule($this->MODULE_ID);
//	    Application::getConnection()->queryExecute('DROP TABLE IF EXISTS ' . \Bitrix\Main\Entity\Base::getInstance('\Litebox\Kassa\Fiscalization\ReceiptTable')->getDBTableName());
//        Option::delete($this->MODULE_ID);
	    CAgent::RemoveModuleAgents("litebox.kassa");

    }

    public function InstallFiles()
    {
        if(\Bitrix\Main\IO\Directory::isDirectoryExists(__DIR__ . $this->componentPath)) {
            CopyDirFiles(__DIR__ . $this->componentPath, $_SERVER["DOCUMENT_ROOT"] . "/" . $this->pathResources . $this->componentPath, true, true);
        } else {
            throw new \Bitrix\Main\IO\InvalidPathException(__DIR__ . $this->componentPath);
        }

        if(\Bitrix\Main\IO\Directory::isDirectoryExists(__DIR__ . $this->apiPath)) {
            CopyDirFiles(__DIR__ . $this->apiPath, $_SERVER["DOCUMENT_ROOT"] . $this->apiPath, true, true);
        } else {
            throw new \Bitrix\Main\IO\InvalidPathException(__DIR__ . $this->apiPath);
        }
    }

    public function InstallDB()
    {
        \Bitrix\Main\Loader::IncludeModule($this->MODULE_ID);
        if (!Application::getConnection()->isTableExists(\Bitrix\Main\Entity\Base::getInstance('\Litebox\Kassa\Fiscalization\ReceiptTable')->getDBTableName()))
        {
	        \Bitrix\Main\Entity\Base::getInstance('\Litebox\Kassa\Fiscalization\ReceiptTable')->createDbTable();
        }

	    CAgent::AddAgent(
		    "\Litebox\Kassa\Payment::updateStatuses();",
		    "litebox.kassa",
		    "N",
		    300,
		    new \Bitrix\Main\Type\DateTime,
		    "Y",
		    new \Bitrix\Main\Type\DateTime,
		    30);
    }

    public function InstallRouter()
    {
        CUrlRewriter::Add($this->urlRules);
    }

    public function UnInstallRouter()
    {
        $sites = CSite::GetList($by="sort", $order="desc");
        $site = $sites->Fetch();
        $this->urlRules['SITE_ID'] = $site['LID'];
        unset($this->urlRules['SORT']);
        unset($this->urlRules['RULE']);
        CUrlRewriter::Delete($this->urlRules);
    }

	public function InstallEvents()
	{
		EventManager::getInstance()->registerEventHandler(
			'sale',
			'OnSalePaymentEntitySaved',
			$this->MODULE_ID,
			'\Litebox\Kassa\Payment',
			'OnSalePaymentEntitySaved'
		);
	}

	public function UnInstallEvents()
	{
		EventManager::getInstance()->unRegisterEventHandler(
			'sale',
			'OnSalePaymentEntitySaved',
			$this->MODULE_ID,
			'\Litebox\Kassa\Payment',
			'OnSalePaymentEntitySaved'
		);
	}
}