<?php
namespace Litebox\Kassa\Fiscalization;

use \Bitrix\Main\Config\Option;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\Web\HttpClient;

Loc::loadMessages(__FILE__);
class Connector
{
	private $ApiToken;
	private $Login;
	private $Password;
	private $ApiUrl;
	private $ShopId;
	private $ModuleId = 'litebox.kassa';

	public function __construct()
	{
		$this->ApiUrl = Option::get($this->ModuleId, "api_url");
		$this->Login = Option::get($this->ModuleId, "api_login");
		$this->Password = Option::get($this->ModuleId, "api_password");
		$this->ApiToken = Option::get($this->ModuleId, "api_token");
		$this->ShopId = Option::get($this->ModuleId, "shop_id");
	}

	public function requestToken()
	{
		$arResult = array();
		if (empty($this->Login) || empty($this->Password))
		{
			$arResult['error'] = Loc::getMessage("LITEBOX_SYNC_CONNECTOR_EMPTY_LOGIN");
			return $arResult;
		}
		if (empty($this->ApiUrl))
		{
			$arResult['error'] = Loc::getMessage("LITEBOX_SYNC_CONNECTOR_EMPTY_URL");
			return $arResult;
		}

		$client = new HttpClient();
		$client->setHeader("Content-type", "application/json", true);
		$client->setHeader("charset", "utf-8", true);

		$data = json_encode([
			'login' => $this->Login,
			'password' => $this->Password
		]);

		$arResponse = $client->post($this->ApiUrl . "/auth", $data);
		$arResponse = json_decode($arResponse, true);

		if (!empty($arResponse['error']))
		{
			$arResult = $arResponse['error']['text'];
		}
		elseif (!empty($arResponse['token']))
		{
			$arResult['token'] = $arResponse['token'];
		}
		else
		{
			$arResult['error'] = Loc::getMessage("LITEBOX_SYNC_CONNECTOR_ERROR_UNDEFINED");
		}

		return $arResult;
	}

	public function sell($data, $paymentId)
	{
		if (empty($this->ApiUrl) || empty($this->ShopId))
		{
			//TODO записать ошибку в лог
			return;
		}

		$client = new HttpClient();
		$client->setHeader("Content-type", "application/json", true);
		$client->setHeader("charset", "utf-8", true);
		$client->setHeader("Authorization", "Token " . $this->ApiToken, true);

		$arResult = $client->post($this->ApiUrl . "/shops/" . $this->ShopId . "/sell", json_encode($data));

		$arResult = json_decode($arResult, true);

		$receipt = \Litebox\Kassa\Fiscalization\ReceiptTable::getRow(array(
			"select" => array("ID"),
			"filter" => array("PAYMENT_ID" => $paymentId, "OPERATION_TYPE" => "sale")
		));
		if (empty($receipt)) {
			if (empty($arResult["error"])) {
				\Litebox\Kassa\Fiscalization\ReceiptTable::add(array(
					"UUID" => $arResult["uuid"],
					"STATUS" => $arResult["status"],
					"OPERATION_TYPE" => 'sell',
					"CREATED" => new \Bitrix\Main\Type\DateTime,
					"LAST_MODIFIED" => new \Bitrix\Main\Type\DateTime,
					"PAYMENT_ID" => $paymentId,
					"ERROR_TEXT" => ""
				));
			} else {
				\Litebox\Kassa\Fiscalization\ReceiptTable::add(array(
					"UUID" => "",
					"STATUS" => "error",
					"OPERATION_TYPE" => 'sell',
					"CREATED" => new \Bitrix\Main\Type\DateTime,
					"LAST_MODIFIED" => new \Bitrix\Main\Type\DateTime,
					"PAYMENT_ID" => $paymentId,
					"ERROR_TEXT" => $arResult['error']['text']
				));
			}
		}
		else
		{
			if (empty($arResult["error"])) {
				\Litebox\Kassa\Fiscalization\ReceiptTable::update($receipt["ID"], array(
					"UUID" => $arResult["uuid"],
					"STATUS" => $arResult["status"],
					"OPERATION_TYPE" => 'sale',
					"LAST_MODIFIED" => new \Bitrix\Main\Type\DateTime,
					"PAYMENT_ID" => $paymentId,
					"ERROR_TEXT" => ""
				));
			} else {
				\Litebox\Kassa\Fiscalization\ReceiptTable::update($receipt["ID"], array(
					"UUID" => "",
					"STATUS" => "error",
					"OPERATION_TYPE" => 'sale',
					"LAST_MODIFIED" => new \Bitrix\Main\Type\DateTime,
					"PAYMENT_ID" => $paymentId,
					"ERROR_TEXT" => $arResult['error']['text']
				));
			}
		}
	}

	public function sellRefund($data, $paymentId)
	{
		if (empty($this->ApiUrl) || empty($this->ShopId) || empty($this->ApiToken))
		{
			//TODO записать ошибку в лог
			return;
		}

		$client = new HttpClient();
		$client->setHeader("Content-type", "application/json", true);
		$client->setHeader("charset", "utf-8", true);
		$client->setHeader("Authorization", "Token " . $this->ApiToken, true);

		$arResult = $client->post($this->ApiUrl . "/shops/" . $this->ShopId . "/sell_refund", json_encode($data));

		$arResult = json_decode($arResult, true);

		$receipt = \Litebox\Kassa\Fiscalization\ReceiptTable::getRow(array(
			"select" => array("ID"),
			"filter" => array("PAYMENT_ID" => $paymentId, "OPERATION_TYPE" => "return")
		));
		if (empty($receipt)) {
			if (empty($arResult["error"])) {
				\Litebox\Kassa\Fiscalization\ReceiptTable::add(array(
					"UUID" => $arResult["uuid"],
					"STATUS" => $arResult["status"],
					"OPERATION_TYPE" => 'return',
					"CREATED" => new \Bitrix\Main\Type\DateTime,
					"LAST_MODIFIED" => new \Bitrix\Main\Type\DateTime,
					"PAYMENT_ID" => $paymentId,
					"ERROR_TEXT" => ""
				));
			} else {
				\Litebox\Kassa\Fiscalization\ReceiptTable::add(array(
					"UUID" => "",
					"STATUS" => "error",
					"OPERATION_TYPE" => 'return',
					"CREATED" => new \Bitrix\Main\Type\DateTime,
					"LAST_MODIFIED" => new \Bitrix\Main\Type\DateTime,
					"PAYMENT_ID" => $paymentId,
					"ERROR_TEXT" => $arResult['error']['text']
				));
			}
		}
		else
		{
			if (empty($arResult["error"])) {
				\Litebox\Kassa\Fiscalization\ReceiptTable::update($receipt["ID"], array(
					"UUID" => $arResult["uuid"],
					"STATUS" => $arResult["status"],
					"OPERATION_TYPE" => 'return',
					"LAST_MODIFIED" => new \Bitrix\Main\Type\DateTime,
					"PAYMENT_ID" => $paymentId,
					"ERROR_TEXT" => ""
				));
			} else {
				\Litebox\Kassa\Fiscalization\ReceiptTable::update($receipt["ID"], array(
					"UUID" => "",
					"STATUS" => "error",
					"OPERATION_TYPE" => 'return',
					"LAST_MODIFIED" => new \Bitrix\Main\Type\DateTime,
					"PAYMENT_ID" => $paymentId,
					"ERROR_TEXT" => $arResult['error']['text']
				));
			}
		}
	}

	public function updateStatus($uuid)
	{
		if (empty($this->ApiUrl) || empty($this->ShopId) || empty($this->ApiToken))
		{
			return;
		}

		$client = new HttpClient();
		$client->setHeader("Content-type", "application/json", true);
		$client->setHeader("charset", "utf-8", true);
		$client->setHeader("Authorization", "Token " . $this->ApiToken, true);

		$arResult = $client->get($this->ApiUrl . "/shops/" . $this->ShopId . "/status/" . $uuid);
		$arResult = json_decode($arResult, true);

		if (empty($arResult["error"]))
		{
			$receipt = \Litebox\Kassa\Fiscalization\ReceiptTable::getRow(array(
				"select" => array("ID"),
				"filter" => array("UUID" => $uuid)
			));
			\Litebox\Kassa\Fiscalization\ReceiptTable::update($receipt["ID"], array(
				"STATUS" => $arResult["status"],
				"LAST_MODIFIED" => new \Bitrix\Main\Type\DateTime,
			));
		}
	}
}