<?php

namespace Litebox\Kassa;

use \Bitrix\Main\Config\Option;
use \Bitrix\Main\Event;

class Payment
{
	public function OnSalePaymentEntitySaved(Event $event)
	{
		$processedPaymentSystems = Option::get('litebox.kassa', "chosen_payment_system");
		$arProcessedPaymentSystems = explode(",", $processedPaymentSystems);

		$payment = $event->getParameter("ENTITY");
		$PaymentSystemId = $payment->getPaymentSystemId();
		if (!in_array($PaymentSystemId, $arProcessedPaymentSystems)) {
			return;
		}
		$isPaid = $payment->isPaid();
		$isReturn = $payment->isReturn();
		$arPaymentFields = $payment->getFieldValues();
		$paymentId = $arPaymentFields["ID"];

		if ($isPaid)
		{
			$receipt = \Litebox\Kassa\Fiscalization\ReceiptTable::getRow(array(
				"select" => array("OPERATION_TYPE", "STATUS"),
				"filter" => array("PAYMENT_ID" => $paymentId, "OPERATION_TYPE" => "sell")
			));
			if (!empty($receipt))
			{
				//Уже есть сохранённый чек без ошибок
				if ($receipt["OPERATION_TYPE"] == "sell" && $receipt["STATUS"] != 'error')
				{
					return;
				}
			}
		}
		elseif ($isReturn)
		{
			$receiptSell = \Litebox\Kassa\Fiscalization\ReceiptTable::getRow(array(
				"select" => array("OPERATION_TYPE", "STATUS"),
				"filter" => array("PAYMENT_ID" => $paymentId, "OPERATION_TYPE" => "sell")
			));
			if (empty($receiptSell))
			{
				return;
			}
			elseif ($receiptSell["STATUS"] == 'error')
			{
				return;
			}

			$receiptReturn = \Litebox\Kassa\Fiscalization\ReceiptTable::getRow(array(
				"select" => array("OPERATION_TYPE", "STATUS"),
				"filter" => array("PAYMENT_ID" => $paymentId, "OPERATION_TYPE" => "return")
			));
			if (!empty($receiptReturn) && $receiptReturn["STATUS"] != 'error')
			{
				return;
			}
		}

		$OrderId = $payment->getOrderId();
		$order = \Bitrix\Sale\Order::load($OrderId);
		$siteId = $order->getSiteId();
		$arSite = \Bitrix\Main\SiteTable::getRowById($siteId);
		$siteEmail = $arSite["EMAIL"];
		$siteUrl = $arSite["SERVER_NAME"];
		$userEmail = $order->getPropertyCollection()->getUserEmail()->getValue();
		$defaultInn = \Bitrix\Sale\BusinessValue::get('SELLER_COMPANY_INN');
		$taxationSystem = Option::get('litebox.kassa', "taxation_system");
		$basket = $order->getBasket();
		$basketItems = $basket->getBasketItems();
		$arProducts = array();
		foreach($basketItems as $basketItem)
		{
			$product = array();
			$product["name"] = $basketItem->getField("NAME");
			$product["price"] = $basketItem->getPrice();
			$product["quantity"] = $basketItem->getQuantity();
			$product["sum"] = $basketItem->getFinalPrice();
			$product["measurement_unit"] = $basketItem->getField("MEASURE_NAME");
			$product["payment_method"] = "full_prepayment";
			$product["payment_object"] = "commodity";
			$vatRate = $basketItem->getField("VAT_RATE");
			if (number_format($vatRate, 2) == 0.00)
			{
				$vatType = "none";
			}
			elseif (number_format($vatRate, 2) == 0.10)
			{
				$vatType = "vat10";
			}
			elseif (number_format($vatRate, 2) == 0.20)
			{
				$vatType = "vat20";
			}
			//Неподдерживаемый тип налога
			//TODO Возможно нужно вывести ошибку
			else
			{
				return;
			}
			$vatValue = ($product["sum"] / ((1 + $vatRate) * 100)) * ($vatRate * 100);
			$product["vat"]["type"] = $vatType;
			$product["vat"]["sum"] = round($vatValue, 2);
			$arProducts[] = $product;
		}
		$data = [
			"external_id" => $OrderId . '_' . time(),
			"timestamp" => date('d.m.Y H:i:s'),
			"receipt" => [
				"client" => [
					"email" => $userEmail
				],
				"company" => [
					"email" => $siteEmail,
					"inn" => $defaultInn,
					"sno" => $taxationSystem,
					"payment_address" => $siteUrl
				],
				"items" => $arProducts,
				"payments" => [
					[
						"type" => "1",
						"sum" => round($arPaymentFields["SUM"], 2)
					]
				],
				"total" => round($arPaymentFields["SUM"], 2)
			]
		];

		$connector = new \Litebox\Kassa\Fiscalization\Connector();
		if ($isPaid)
		{
			$connector->sell($data, $paymentId);
		}
		elseif ($isReturn)
		{
			$connector->sellRefund($data, $paymentId);
		}
	}

	public static function updateStatuses()
	{
		$receipts = \Litebox\Kassa\Fiscalization\ReceiptTable::getList(array(
			"select" => array("UUID"),
			"filter" => array("STATUS" => "wait")
		));

		$connector = new \Litebox\Kassa\Fiscalization\Connector();
		while ($receipt = $receipts->fetch())
		{
			$connector->updateStatus($receipt['UUID']);
		}

		return "\Litebox\Kassa\Payment::updateStatuses();";
	}
}