<?php
use \Bitrix\Main\Localization;
use \Bitrix\Main\Localization\Loc;
use \Bitrix\Main\Loader;
use \Bitrix\Main\Config\Option;
use Bitrix\Sale\PaySystem\Manager;

Loc::loadMessages($_SERVER["DOCUMENT_ROOT"].BX_ROOT . "/modules/main/options.php");
Loc::loadMessages(__FILE__);
$module_id = "litebox.kassa";
$RIGHT = $APPLICATION->GetGroupRight($module_id);

CAdminNotify::DeleteByTag("litebox_error");
$request = \Bitrix\Main\HttpApplication::getInstance()->getContext()->getRequest();
Loader::includeModule('sale');

if ($RIGHT >= "R") :
    $arPaySystems = array();
    $paySystemResult = Manager::getList(array(
        'filter'  => array(
            'ACTIVE' => 'Y',
        )
    ));
    while ($paySystem = $paySystemResult->fetch())
    {
        $arPaySystems[$paySystem["ID"]] = $paySystem["NAME"];
    }
    $aTabs = [
        ["DIV" => "edit1", "ICON" => "perfmon_settings",
            "TAB" => Loc::getMessage('LITEBOX_SYNC_TAB_DOCUMENTATION'),
            "TITLE" => Loc::getMessage('LITEBOX_SYNC_TAB_DOCUMENTATION_TITLE')],
        ["DIV" => "edit2", "ICON" => "perfmon_settings",
            "TAB" => Loc::getMessage('LITEBOX_SYNC_TAB_GROUP_RIGHTS'),
            "TITLE" => Loc::getMessage('LITEBOX_SYNC_TAB_GROUP_RIGHTS_TITLE')],
        ["DIV" => "edit3", "ICON" => "perfmon_settings",
            "TAB" => Loc::getMessage('LITEBOX_SYNC_TAB_FISCALIZATION'),
            "TITLE" => Loc::getMessage('LITEBOX_SYNC_TAB_FISCALIZATION_TITLE'),
            "OPTIONS" => [
                [
                    "api_url",
                    Loc::getMessage("LITEBOX_SYNC_API_URL"),
                    "",
                    ["text", 50]
                ],
                [
                    "api_login",
                    Loc::getMessage("LITEBOX_SYNC_API_LOGIN"),
                    "",
                    ["text", 40]
                ],
                [
                    "api_password",
                    Loc::getMessage("LITEBOX_SYNC_API_PASSWORD"),
                    "",
                    ["password", 40]
                ],
                [
                    "api_token",
                    Loc::getMessage("LITEBOX_SYNC_API_TOKEN"),
                    "",
                    ["text", 100]
                ],
                [
                    "shop_id",
                    Loc::getMessage("LITEBOX_SYNC_SHOP_ID_SETTING"),
                    "",
                    ["text", 20]
                ],
                [
                    "chosen_payment_system",
                    Loc::getMessage("LITEBOX_SYNC_CHOSEN_PAYMENT_SYSTEM"),
                    "",
                    ["multiselectbox", $arPaySystems]
                ],
                [
                    "taxation_system",
                    Loc::getMessage("LITEBOX_SYNC_CHOSEN_TAXATION_SYSTEM"),
                    "",
                    //TODO Ленговые переменные
                    [
                        "selectbox",
                        [
                            "osn" => "Общая СН",
                            "usn_income" => "Упрощённая СН (доходы)",
                            "usn_income_outcome" => "Упрощённая СН (доходы минус расходы)",
                            "envd" => "Единый налог на вменённый доход",
                            "esn" => "Единый сельскохозяйственный налог",
                            "patent" => "Патентная СН"
                        ]
                    ]
                ]
            ]
        ],
        ["DIV" => "edit4", "ICON" => "perfmon_settings",
            "TAB" => Loc::getMessage('LITEBOX_SYNC_TAB_CATALOG'),
            "TITLE" => Loc::getMessage('LITEBOX_SYNC_TAB_CATALOG_TITLE'),
            "OPTIONS" => [
                [
                    "ib_catalog_id",
                    Loc::getMessage("LITEBOX_SYNC_IB_CATALOG_ID_LABEL"),
                    "catalog",
                    ["text", 50]
                ],
                [
                    "catalog_id",
                    Loc::getMessage("LITEBOX_SYNC_CATALOG_ID_LABEL"),
                    "",
                    ["text", 50]
                ],
                [
                    'images_uploading_enable',
                    Loc::getMessage('LITEBOX_SYNC_IMAGES_UPLOADING_ENABLE_LABEL'),
                    'Y',
                    ['checkbox']
                ],
                [
                    "vendorcode_product_prop_code",
                    Loc::getMessage("LITEBOX_SYNC_VENDORCODE_PRODUCT_PROP_LABEL"),
                    "", // ARTNUMBER
                    ["text", 50]
                ],
                [
                    "vendorcode_offer_prop_code",
                    Loc::getMessage("LITEBOX_SYNC_VENDORCODE_OFFER_PROP_LABEL"),
                    "", // ARTNUMBER
                    ["text", 50]
                ],
            ],
        ],
    ];

    $tabControl = new CAdminTabControl("tabControl", $aTabs);

    $procTabs = [
        ['DIV' => 'proc_edit0', 'ICON' => '',
            'TAB' => Loc::getMessage('LITEBOX_SYNC_GET_API_TOKEN_TAB'),
            'TITLE' => Loc::getMessage('LITEBOX_SYNC_GET_API_TOKEN_TAB_TITLE')
        ],
    ];
    $procTabControl = new CAdminTabControl("ProcTabControl", $procTabs, true, true);

    Loader::includeModule($module_id);
    $module_path = '/upload/litebox/';

    // Обрабатываем данные после отправки формы
    if ($request->isPost() && $request["Update"] && check_bitrix_sessid()) {
        foreach ($aTabs as $aTab) { // цикл по вкладкам
            foreach ($aTab['OPTIONS'] as $arOption) {
                if (!is_array($arOption)) // если это название секции
                    continue;

                if ($arOption['note']) // если это примечание
                    continue;

                $optionName = $arOption[0];
                $optionValue = $request->getPost($optionName);

                if ($arOption[3][0] == 'checkbox' && $optionValue != 'Y') { // если чекбокс сброшен, то ставим значение 'N'
                    $optionValue = 'N';
                }
                
                Option::set($module_id, $optionName, is_array($optionValue) ? implode(",", $optionValue) : $optionValue);
            }
        }

        LocalRedirect($APPLICATION->GetCurPage().'?lang='.LANGUAGE_ID.'&mid='.$module_id.'&'.$tabControl->ActiveTabParam());
    }

    if ($request->isPost() && isset($request["action"]) && $request["action"] == "get_token" && check_bitrix_sessid())
    {
        $connector = new \Litebox\Kassa\Fiscalization\Connector();
        $arRes = $connector->requestToken();
        if (!empty($arRes['error']))
        {
            CAdminNotify::Add(array(
                'MESSAGE' => $arRes['error'],
                'TAG' => 'litebox_error',
                'MODULE_ID' => $module_id,
                'ENABLE_CLOSE' => 'Y',)
            );
            LocalRedirect($APPLICATION->GetCurPage().'?lang='.LANGUAGE_ID.'&mid='.$module_id.'&'.$procTabControl->ActiveTabParam());
        }
        Option::set($module_id, "api_token", $arRes['token']);
        LocalRedirect($APPLICATION->GetCurPage().'?lang='.LANGUAGE_ID.'&mid='.$module_id.'&tabControl_active_tab=edit3');
    }

    ?>
    <form method="post" enctype="multipart/form-data" action="<?echo $APPLICATION->GetCurPage()?>?mid=<?=urlencode($module_id)?>&amp;lang=<?=LANGUAGE_ID?>">
        <?php
        $tabControl->Begin();


        $tabControl->BeginNextTab(); ?>
        <h2>Полная синхронизация заказов, номенклатуры и остатков интернет-магазина с МТС Кассой и ее товароучетной системой</h2>
        <p>
            В состав модуля входит комплексный компонент, который содержит в себе роутинг по api и библиотеки скриптов.<br>
            Типизация данных происходит под формат json<br>
            После установки модуля будет скопирована папка в корень проекта /api/ с подключением компонента<br>
            API поддерживает версионность<br>
            Проверка работоспособности:  domain/api/v1/token - ответ json с токеном
        </p>


        <? $tabControl->BeginNextTab(); ?>
        <? require_once($_SERVER["DOCUMENT_ROOT"]."/bitrix/modules/main/admin/group_rights.php"); ?>


        <? $tabControl->BeginNextTab(); ?>
        <? __AdmSettingsDrawList($module_id, $aTabs[2]['OPTIONS']); ?>


        <? $tabControl->BeginNextTab(); ?>
        <? __AdmSettingsDrawList($module_id, $aTabs[3]['OPTIONS']); ?>


        <? $tabControl->Buttons(); ?>
        <input <? if ($RIGHT<"W") echo "disabled" ?> type="submit" name="Update" value="<?= Loc::getMessage("MAIN_SAVE") ?>" title="<?= Loc::getMessage("MAIN_OPT_SAVE_TITLE") ?>" class="adm-btn-save">
        <input <? if ($RIGHT<"W") echo "disabled" ?> type="submit" name="Apply" value="<?= Loc::getMessage("MAIN_OPT_APPLY") ?>" title="<?= Loc::getMessage("MAIN_OPT_APPLY_TITLE") ?>">
        <? if (strlen($_REQUEST["back_url_settings"]) > 0):?>
            <input <?if ($RIGHT<"W") echo "disabled" ?> type="button" name="Cancel" value="<?= Loc::getMessage("MAIN_OPT_CANCEL") ?>" title="<?= Loc::getMessage("MAIN_OPT_CANCEL_TITLE") ?>" onclick="window.location='<?echo htmlspecialcharsbx(CUtil::addslashes($_REQUEST["back_url_settings"]))?>'">
            <input type="hidden" name="back_url_settings" value="<?=htmlspecialcharsbx($_REQUEST["back_url_settings"])?>">
        <? endif ?>
        <input type="submit" name="RestoreDefaults" title="<?= Loc::getMessage("MAIN_HINT_RESTORE_DEFAULTS") ?>" OnClick="confirm('<?= Loc::getMessage("MAIN_HINT_RESTORE_DEFAULTS_WARNING") ?>')" value="<?= Loc::getMessage("MAIN_RESTORE_DEFAULTS") ?>">


        <?= bitrix_sessid_post(); ?>
        <? $tabControl->End();?>
    </form>

    <h2><? echo Loc::getMessage('LITEBOX_SYNC_GET_API_TOKEN_FORM_TITLE'); ?></h2>
    <? $procTabControl->Begin(); ?>
    <? $procTabControl->BeginNextTab(); ?>
    <form method="post" action="<?echo $APPLICATION->GetCurPage()?>?mid=<?=urlencode($module_id)?>&amp;lang=<?=LANGUAGE_ID?>" name="api_token">
        <? echo bitrix_sessid_post(); ?>
        <input type="hidden" name="action" value="get_token">
        <input type="submit" <?if ($RIGHT<"W") echo "disabled" ?> name="recalc" value="<?echo Loc::getMessage('LITEBOX_SYNC_GET_API_TOKEN_BUTTON_TEXT');?>">
    </form>
    <? $procTabControl->End() ?>
    <?
    if (!empty($arNotes)) {
        echo BeginNote();
        foreach($arNotes as $i => $str)
        {
            ?><span class="required"><sup><?echo $i+1?></sup></span><?echo $str?><br><?
        }
        echo EndNote();
    }
    ?>
<?endif;?>